@@ -276,8 +276,9 @@ class CursorWrapper(object):
     def execute_select(self, soql: str, args) -> None:
         if soql != MIGRATIONS_QUERY_TO_BE_IGNORED:
             # normal query
-            is_query_all = self.query and self.query.is_query_all
-            self.cursor.execute(soql, args, query_all=is_query_all)
+            query_all = self.query and self.query.sf_params.query_all
+            tooling_api = self.query and self.query.model._meta.sf_tooling_api_model
+            self.cursor.execute(soql, args, query_all=query_all, tooling_api=tooling_api)
         else:
             # Nothing queried about django_migrations to SFDC and immediately responded that
             # nothing about migration status is recorded in SFDC.
@@ -305,7 +306,8 @@ class CursorWrapper(object):
         if self.db.connection.composite_type == 'sobject-collections':
             # SObject Collections
             records = [merge_dict(x, type_=table) for x in post_data]
-            ret = self.db.connection.sobject_collections_request('POST', records)
+            all_or_none = query.sf_params.all_or_none
+            ret = self.db.connection.sobject_collections_request('POST', records, all_or_none=all_or_none)
             self.lastrowid = ret
             self.rowcount = len(ret)
             return
@@ -334,7 +336,9 @@ class CursorWrapper(object):
                 assert not child.bilateral_transforms
                 if isinstance(pks, (tuple, list)):
                     return pks
-                assert isinstance(pks, Query) and type(pks).__name__ == 'SalesforceQuery', (
+                # 'sf_params' are also in 'pks' only in Django >= 2.0, therefore check query.sf_params
+                assert (isinstance(pks, Query) and type(pks).__name__ == 'SalesforceQuery' or
+                        query.sf_params.edge_updates), (
                     "Too complicated queryset.update(). Rewrite it by two querysets. "
                     "See docs wiki/error-messages")
                 # # alternative solution:
@@ -352,7 +356,33 @@ class CursorWrapper(object):
             assert len(cur.description) == 1 and cur.description[0][0] == 'Id'
             return [x[0] for x in cur]
 
+    def execute_tooling_update(self, query):
+        table = query.model._meta.db_table
+        post_data = extract_values(query)
+        pks = self.get_pks_from_query(query)
+        assert len(pks) == 1
+        pk = pks[0]
+        value_map = {qfield.db_column: value for qfield, _, value in query.values}
+        if 'Metadata' in value_map and 'FullName' in value_map and 'DurableId' in value_map:
+            ret = self.db.connection.handle_api_exceptions(
+                'PATCH',
+                'tooling/sobjects', table, value_map['DurableId'],
+                json={'Metadata': value_map['Metadata'], 'FullName': value_map['FullName']}
+            )
+        elif pk == '000000000000000AAA':
+            pks = value_map['DurableId']
+            post_data = dict(**{"attributes": {"type": query.model._meta.db_table}}, **post_data)
+            obj_url = self.db.connection.rest_api_url('tooling/sobjects', table, value_map['DurableId'], relative=True)
+            ret = self.db.connection.handle_api_exceptions('PATCH', obj_url, json=value_map)
+        else:
+            obj_url = self.db.connection.rest_api_url('tooling/sobjects', table, pk, relative=True)
+            ret = self.db.connection.handle_api_exceptions('PATCH', obj_url, json=post_data)
+        assert ret.status_code == 204
+        self.rowcount = 1
+
     def execute_update(self, query):
+        if query.model._meta.sf_tooling_api_model:
+            return self.execute_tooling_update(query)
         table = query.model._meta.db_table
         post_data = extract_values(query)
         pks = self.get_pks_from_query(query)
@@ -368,7 +398,8 @@ class CursorWrapper(object):
         if self.db.connection.composite_type == 'sobject-collections':
             # SObject Collections
             records = [merge_dict(post_data, id=pk, type_=table) for pk in pks]
-            ret = self.db.connection.sobject_collections_request('PATCH', records)
+            all_or_none = query.sf_params.all_or_none
+            ret = self.db.connection.sobject_collections_request('PATCH', records, all_or_none=all_or_none)
             self.lastrowid = ret
             self.rowcount = len(ret)
             return
@@ -394,7 +425,8 @@ class CursorWrapper(object):
         if self.db.connection.composite_type == 'sobject-collections':
             # SObject Collections
             records = pks
-            ret = self.db.connection.sobject_collections_request('DELETE', records)
+            all_or_none = None  # sf_params not supported by DeleteQuery
+            ret = self.db.connection.sobject_collections_request('DELETE', records, all_or_none=all_or_none)
             self.lastrowid = ret
             self.rowcount = len(ret)
             return
@@ -417,7 +449,7 @@ class CursorWrapper(object):
         request (insert).
         """
         ret = self.handle_api_exceptions('GET', '')
-        return str_dict(ret.json())
+        return ret.json()
 
     def id_request(self):
         """The Force.com Identity Service (return type dict of str)"""
@@ -433,8 +465,7 @@ class CursorWrapper(object):
 
     def versions_request(self):
         """List Available REST API Versions"""
-        ret = self.handle_api_exceptions('GET', '', api_ver='')
-        return [str_dict(x) for x in ret.json()]
+        return self.handle_api_exceptions('GET', '', api_ver='').json()
 
     def query_results(self, results):
         while True:
@@ -483,11 +514,6 @@ class CursorWrapper(object):
         return self.cursor.handle_api_exceptions(method, *url_parts, **kwargs)
 
 
-def str_dict(some_dict):
-    """Convert dict of ascii str/unicode to dict of str, if necessary"""
-    return {str(k): str(v) for k, v in some_dict.items()}
-
-
 def sobj_id(obj):
     assert obj._salesforce_object  # pylint:disable=protected-access
     return obj.pk
