@@ -30,7 +30,7 @@ import requests
 from requests.adapters import HTTPAdapter
 
 import salesforce
-from salesforce.auth import SalesforcePasswordAuth, SalesforceAuth
+from salesforce.auth import SalesforceAuth, time_statistics
 from salesforce.dbapi import get_max_retries
 from salesforce.dbapi import settings  # i.e. django.conf.settings
 from salesforce.dbapi.exceptions import (  # NOQA pylint: disable=unused-import
@@ -38,7 +38,7 @@ from salesforce.dbapi.exceptions import (  # NOQA pylint: disable=unused-import
     InternalError, ProgrammingError, NotSupportedError, SalesforceError, SalesforceWarning,
     warn_sf,
     FakeReq, FakeResp, GenResponse)
-from salesforce.dbapi.subselect import nz, QQuery, _TRow
+from salesforce.dbapi.subselect import QQuery, _TRow
 
 try:
     import beatbox  # type: ignore[import]  # pylint: disable=unused-import
@@ -127,10 +127,12 @@ class RawConnection:
         self.messages = []           # type: List[ErrInfo]
 
         self._sf_session = None      # type: Optional[SfSession]
-        self.api_ver = salesforce.API_VERSION
+        self._api_version = settings_dict.get('API_VERSION', salesforce.API_VERSION)  # type: str
         self.debug_verbs = None      # type: Optional[List[str]]
         self.composite_type = 'sobject-collections'  # 'sobject-collections' or 'composite'
 
+        self.sf_auth = SalesforceAuth.create_subclass_instance(db_alias=self.alias,
+                                                               settings_dict=self.settings_dict)
         # The SFDC database is connected as late as possible if only tests
         # are running. Some tests don't require a connection.
         if not getattr(settings, 'SF_LAZY_CONNECT', 'test' in sys.argv):  # TODO don't use argv
@@ -141,6 +143,15 @@ class RawConnection:
     # Methods close() and commit() can be safely ignored, because everything is
     # committed automatically and the REST API is stateless.
 
+    @property
+    def api_ver(self) -> str:
+        # The highest supported SFDC API version can be set by
+        # settings.DATABASES['salesforce']['API_VERSION'] = 'MAX'
+        # It is useful for development of 'inspectdb'
+        if self._api_version == 'MAX':
+            self._api_version = self.handle_api_exceptions('GET', '', api_ver='').json()[-1]['version']
+        return self._api_version
+
     def close(self) -> None:
         del self.messages[:]
         if self._sf_session:
@@ -175,17 +186,16 @@ class RawConnection:
         """Authenticate and get the name of assigned SFDC data server"""
         with connect_lock:
             if self._sf_session is None:
+                auth_data = self.sf_auth.authenticate_and_cache()
+                sf_instance_url = auth_data.get('instance_url')
                 sf_session = SfSession()
-                # TODO configurable class Salesforce***Auth
-                sf_session.auth = SalesforcePasswordAuth(db_alias=self.alias,
-                                                         settings_dict=self.settings_dict)
-                sf_instance_url = sf_session.auth.instance_url
-                if sf_instance_url not in sf_session.adapters:
+                sf_session.auth = self.sf_auth
+                if sf_instance_url and sf_instance_url not in sf_session.adapters:
                     # a repeated mount to the same prefix would cause a warning about unclosed SSL socket
                     sf_requests_adapter = HTTPAdapter(max_retries=get_max_retries())
                     sf_session.mount(sf_instance_url, sf_requests_adapter)
                 # Additional headers work, but the same are added automatically by "requests' package.
-                # sf_session.header = {'accept-encoding': 'gzip, deflate', 'connection': 'keep-alive'} # TODO
+                # sf_session.header = {'accept-encoding': 'gzip, deflate', 'connection': 'keep-alive'}
                 self._sf_session = sf_session
 
     def rest_api_url(self, *url_parts_: str, **kwargs: Any) -> str:
@@ -220,7 +230,7 @@ class RawConnection:
         api_ver = api_ver if api_ver is not None else self.api_ver
         assert not kwargs
         if not relative:
-            base = [self.sf_session.auth.instance_url]
+            base = [self.sf_auth.instance_url]
         else:
             base = ['']
         if url_parts and url_parts[0].startswith('/'):
@@ -279,11 +289,12 @@ class RawConnection:
             response = session.request(method, url, **kwargs_in)
         except requests.exceptions.Timeout:
             raise SalesforceError("Timeout, URL=%s" % url)
-        if response.status_code == 401:  # Unauthorized
+        if (response.status_code == 401                      # Unauthorized
+                and 'json' in response.headers['content-type']
+                and response.json()[0]['errorCode'] == 'INVALID_SESSION_ID'):
             # Reauthenticate and retry (expired or invalid session ID or OAuth)
-            if ('json' in response.headers['content-type']
-                    and response.json()[0]['errorCode'] == 'INVALID_SESSION_ID'):
-                token = session.auth.reauthenticate()
+            token = session.auth.reauthenticate()
+            if token:
                 if 'headers' in kwargs:
                     kwargs['headers'].update(Authorization='OAuth %s' % token)
                 try:
@@ -340,6 +351,15 @@ class RawConnection:
         # it is good e.g for these errorCode: ('INVALID_FIELD', 'MALFORMED_QUERY', 'INVALID_FIELD_FOR_INSERT_UPDATE')
         raise SalesforceError([err_msg], response)
 
+    def handle_api_exceptions_big(self, method: str, *url_parts: str, **kwargs: Any) -> requests.Response:
+        """Call REST API with the query encapsulated into the body if the query is big"""
+        assert method == 'GET'
+        api_ver = kwargs.pop('api_ver', None)
+        url = self.rest_api_url(*url_parts, api_ver=api_ver)
+        url = re.sub(r'^\w+://[^/]+', '', url)
+        data = [{'method': 'GET', 'url': url, 'referenceId': 'subrequest_0'}]
+        return self.composite_request(data)
+
     def composite_request(self, data: List[Dict[str, Any]]) -> requests.Response:
         """Call a 'composite' request with subrequests, error handling
 
@@ -459,7 +479,10 @@ class RawConnection:
         Returns the duration if the command succeded.
         """
         t_0 = time.time()
-        self.handle_api_exceptions('GET', '', api_ver='', timeout=timeout)
+        try:
+            self.handle_api_exceptions('GET', '', api_ver='', timeout=timeout)
+        except (requests.exceptions.RequestException, SalesforceError):
+            pass
         return round(time.time() - t_0, 3)
 
 
@@ -532,13 +555,15 @@ class Cursor(Generic[_TRow]):
     def close(self) -> None:
         self._clean()
 
-    def execute(self, soql: str, parameters: Optional[Iterable[Any]] = None, query_all: bool = False) -> None:
+    def execute(self, soql: str, parameters: Optional[Iterable[Any]] = None, query_all: bool = False,
+                tooling_api: bool = False) -> None:
         self._clean()
         parameters = parameters or []
         sqltype = soql.split(None, 1)[0].upper()
         if sqltype == 'SELECT':
-            self.execute_select(soql, parameters, query_all=query_all)
+            self.execute_select(soql, parameters, query_all=query_all, tooling_api=tooling_api)
         elif sqltype == 'EXPLAIN':
+            assert not tooling_api
             self.execute_explain(soql, parameters, query_all=query_all)
         else:
             # INSERT UPDATE DELETE EXPLAIN
@@ -621,9 +646,11 @@ class Cursor(Generic[_TRow]):
             self.query_more(self._next_records_url)
             self._chunk_offset = new_offset
 
-    def execute_select(self, soql: str, parameters: Iterable[Any], query_all: bool = False) -> None:
+    def execute_select(self, soql: str, parameters: Iterable[Any], query_all: bool = False,
+                       tooling_api: bool = False) -> None:
         processed_sql = str(soql) % tuple(arg_to_soql(x) for x in parameters)
-        service = 'query' if not query_all else 'queryAll'
+        service = '' if not tooling_api else 'tooling/'
+        service += 'query' if not query_all else 'queryAll'
 
         self.qquery = qquery = QQuery(soql)
         # TODO better description
@@ -658,7 +685,11 @@ class Cursor(Generic[_TRow]):
 
     def query_more(self, nextRecordsUrl: str) -> None:  # pylint:disable=invalid-name
         self._check()
-        ret = self.handle_api_exceptions('GET', nextRecordsUrl).json()
+        if len(nextRecordsUrl) < 15500:
+            ret = self.handle_api_exceptions('GET', nextRecordsUrl).json()
+        else:
+            ret = self.connection.handle_api_exceptions_big('GET', nextRecordsUrl).json()
+            ret = ret['compositeResponse'][0]['body']
         self.rowcount = ret['totalSize']  # may be more accurate than the initial approximate value
         self._chunk = ret['records']
         self._next_records_url = ret.get('nextRecordsUrl')
@@ -857,27 +888,3 @@ def merge_dict(dict_1: Dict[Any, Any], *other: Dict[Any, Any], **kw: Any) -> Dic
         tmp.update(x)
     tmp.update(kw)
     return tmp
-
-
-class TimeStatistics:
-
-    def __init__(self, expiration: float = 300) -> None:
-        self.expiration = expiration
-        self.data = {}  # type: Dict[str, float]
-
-    def update_callback(self, url: str, callback: Optional[Callable[[], Any]] = None) -> None:
-        """Update the statistics for the domain"""
-        domain = self.domain(url)
-        last_req = self.data.get(domain, 0)
-        t_new = time.time()
-        do_call = (t_new - last_req > self.expiration)
-        self.data[domain] = t_new
-        if do_call and callback:
-            callback()
-
-    @staticmethod
-    def domain(url: str) -> str:
-        return nz(re.match(r'^(?:https|mock)://([^/]*)/?', url)).groups()[0]
-
-
-time_statistics = TimeStatistics(300)
