@@ -67,25 +67,29 @@ class SalesforceModelBase(ModelBase):
             result._meta.original_attrs['db_table'] = result._meta.db_table
         return result
 
-    def add_to_class(cls, name, value):  # type: ignore[no-untyped-def] # noqa
+    def add_to_class(cls, name: str, value: Any) -> Any:
         # pylint:disable=protected-access
         if name == '_meta':
             sf_custom = False
+            sf_tooling_api_model = False
             if hasattr(value.meta, 'custom'):
                 sf_custom = value.meta.custom
                 delattr(value.meta, 'custom')
+            if hasattr(value.meta, 'sf_tooling_api_model'):
+                sf_tooling_api_model = value.meta.sf_tooling_api_model
+                delattr(value.meta, 'sf_tooling_api_model')
             super(SalesforceModelBase, cls).add_to_class(name, value)  # type: ignore[misc] # noqa
             setattr(cls._meta, 'sf_custom', sf_custom)  # type: ignore[attr-defined] # noqa
+            setattr(cls._meta, 'sf_tooling_api_model', sf_tooling_api_model)  # type: ignore[attr-defined] # noqa
         else:
-            if not TYPE_CHECKING:
-                if type(value) is models.manager.Manager:  # pylint:disable=unidiomatic-typecheck
-                    # this is for better migrations because: obj._constructor_args = (args, kwargs)
-                    _constructor_args = value._constructor_args
-                    value = models.manager.Manager()
-                    # value = manager.SalesforceManager()
-                    value._constructor_args = _constructor_args
+            if type(value) is models.manager.Manager:  # pylint:disable=unidiomatic-typecheck
+                # this is for better migrations because: obj._constructor_args = (args, kwargs)
+                _constructor_args = value._constructor_args
+                value = models.manager.Manager()
+                # value = manager.SalesforceManager()
+                value._constructor_args = _constructor_args
 
-                super(SalesforceModelBase, cls).add_to_class(name, value)
+            super(SalesforceModelBase, cls).add_to_class(name, value)  # type: ignore[misc] # noqa
 
 
 if TYPE_CHECKING:
@@ -107,11 +111,15 @@ else:
         # pylint:disable=invalid-name
         _salesforce_object = 'standard'
 
+        # If both managers are specified in this class, they should be different objects,
+        # otherwse the name of the base manager must be passed to the mataclass before
+        # the name of default manager, but that order is randomized by Python <= 3.5.x
+        base_manager = manager.SalesforceManager()  # type: manager.SalesforceManager[_T]
         objects = manager.SalesforceManager()  # type: manager.SalesforceManager[_T]
 
         class Meta:
             abstract = True
-            base_manager_name = 'objects'
+            base_manager_name = 'base_manager'
             if not DJANGO_20_PLUS:
                 manager_inheritance_from_future = True
 
