@@ -1153,6 +1153,17 @@ class BasicSOQLRoTest(TestCase, LazyTestMixin):
         self.assertTrue(models_template)
         self.assertIn('@', Organization.objects.get().created_by.Username)
 
+    def test_big_soql(self):
+        """Test that a query of length almost 100000 is possible"""
+        contact = Contact.objects.all()[0]
+        # 4750 items * 21 characters == 99750
+        qs = Contact.objects.filter(pk__in=4750 * [contact.pk])
+        self.assertEqual(list(qs.values_list('pk', flat=True)), [contact.pk])
+
+    def test_empty_slice(self):
+        """Test queryset with empty slice - if high/low limits equals"""
+        self.assertEqual(len(Contact.objects.all()[1:1]), 0)
+
 # ============= Tests that need setUp Lead ==================
 
 
